/*
 * getrlimit
 *    Uses getrlimit(2) to get resource limits
 *
 * (!c) 2002 by Ighighi
 * Venezuela
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/resource.h>
#include <errno.h>

static void print_rlimit(const char *s, int name);

#define do_it(name)	print_rlimit(#name, name)

static void print_rlimit(const char *s, int name)
{
    struct rlimit limit;

    if (getrlimit(name, &limit) < 0) {
	fprintf(stderr, "Error: ");
	perror(s);
    }
    else {
	printf("%-20s    ", s);
	if (limit.rlim_cur == RLIM_INFINITY)
	    printf("%15s    ", "infinite");
	else
	    printf("%15ld    ", limit.rlim_cur);
	if (limit.rlim_max == RLIM_INFINITY)
	    printf("%15s    \n", "infinite");
	else
	    printf("%15ld    \n", limit.rlim_max);
    }
    
    return;
}

int main(void)
{

    printf("%-20s    %15s    %15s\n", "resource", "soft limit", "hard limit");
    
#ifdef RLIMIT_CPU
    do_it(RLIMIT_CPU);
#endif

#ifdef RLIMIT_FSIZE
    do_it(RLIMIT_FSIZE);
#endif

#ifdef RLIMIT_DATA
    do_it(RLIMIT_DATA);
#endif

#ifdef RLIMIT_STACK
    do_it(RLIMIT_STACK);
#endif

#ifdef RLIMIT_CORE
    do_it(RLIMIT_CORE);
#endif

#ifdef RLIMIT_RSS
    do_it(RLIMIT_RSS);
#endif

#ifdef RLIMIT_NOFILE
    do_it(RLIMIT_NOFILE);
#endif

#ifdef RLIMIT_OFILE
    do_it(RLIMIT_OFILE);
#endif

#ifdef RLIMIT_AS
    do_it(RLIMIT_AS);
#endif

#ifdef RLIMIT_NPROC
    do_it(RLIMIT_NPROC);
#endif

#ifdef RLIMIT_MEMLOCK
    do_it(RLIMIT_MEMLOCK);
#endif

#ifdef RLIMIT_LOCKS
    do_it(RLIMIT_LOCKS);
#endif

    return 0;
}
