/*
 * pathconf
 *    Uses pathconf(3) to get configuration values for files
 *
 * (!c) 2002 by Ighighi
 * Venezuela
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

static void print_pathconf(const char *s, const char *path, int name);

#define do_it(name, path)	print_pathconf(#name, path, name)

static void print_pathconf(const char *s, const char *path, int name)
{
    long value;

    errno = 0;
    value = pathconf(path, name);
    if (value < 0) {
	if (errno) {
	    fprintf(stderr, "Error: ");
	    perror(s);
	}
	else {
	    printf("%-32s    ", s);
	    printf("%15s\n", "unlimited");
	}
    }
    else {
	printf("%-32s    ", s);    	    	
	printf("%15ld\n", value);
    }
    
    return;
}

int main(int argc, char *argv[])
{
    char *path;
    
    path = argv[1];    
    if (! path) {
	fprintf(stderr, "Usage: %s pathname\n", argv[0]);
	exit(1);
    }
    
    printf("%-32s    %15s\n", "name", "value");
    
#ifdef _PC_LINK_MAX
    do_it(_PC_LINK_MAX, path);
#endif

#ifdef _PC_MAX_CANON
    do_it(_PC_MAX_CANON, path);
#endif

#ifdef _PC_MAX_INPUT
    do_it(_PC_MAX_INPUT, path);
#endif

#ifdef _PC_NAME_MAX
    do_it(_PC_NAME_MAX, path);
#endif

#ifdef _PC_PATH_MAX
    do_it(_PC_PATH_MAX, path);
#endif

#ifdef _PC_PIPE_BUF
    do_it(_PC_PIPE_BUF, path);
#endif

#ifdef _PC_CHOWN_RESTRICTED
    do_it(_PC_CHOWN_RESTRICTED, path);
#endif

#ifdef _PC_NO_TRUNC
    do_it(_PC_NO_TRUNC, path);
#endif

#ifdef _PC_VDISABLE
    do_it(_PC_VDISABLE, path);
#endif

#ifdef _PC_SYNC_IO
    do_it(_PC_SYNC_IO, path);
#endif

#ifdef _PC_ASYNC_IO
    do_it(_PC_ASYNC_IO, path);
#endif

#ifdef _PC_PRIO_IO
    do_it(_PC_PRIO_IO, path);
#endif

#ifdef _PC_SOCK_MAXBUF
    do_it(_PC_SOCK_MAXBUF, path);
#endif

#ifdef _PC_FILESIZEBITS
    do_it(_PC_FILESIZEBITS, path);
#endif

#ifdef _PC_REC_INCR_XFER_SIZE
    do_it(_PC_REC_INCR_XFER_SIZE, path);
#endif

#ifdef _PC_REC_MAX_XFER_SIZE
    do_it(_PC_REC_MAX_XFER_SIZE, path);
#endif

#ifdef _PC_REC_MIN_XFER_SIZE
    do_it(_PC_REC_MIN_XFER_SIZE, path);
#endif

#ifdef _PC_REC_XFER_ALIGN
    do_it(_PC_REC_XFER_ALIGN, path);
#endif

#ifdef _PC_ALLOC_SIZE_MIN
    do_it(_PC_ALLOC_SIZE_MIN, path);
#endif

#ifdef _PC_SYMLINK_MAX
    do_it(_PC_SYMLINK_MAX, path);
#endif

    return 0;
}
