/*
 * stat:
 *   Print inode contents
 * 
 * (!c) 2001, 2002 by Ighighi
 * Venezuela
 * 
 * TODO:
 *    XXX's
 *    BSD's (u_int32_t) st_flags & (u_int32_t) st_gen
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <pwd.h>
#include <grp.h>

#include <limits.h>		/* PATH_MAX */
#include <sys/param.h>		/* MAXPATHLEN (BSD) */

/*
 * XXX - major() & minor(): <sys/types.h> (BSD), <sys/sysmacros.h> (SVR4)
 */
#ifndef	major
#include <sys/sysmacros.h>
#endif

#ifdef	S_IFLNK	/* XXX */
#define STAT	lstat
#else
#define STAT	stat
#endif

#ifndef PATH_MAX
#ifdef	MAXPATHLEN
#define PATH_MAX	MAXPATHLEN
#else
#define PATH_MAX	1024
#endif
#endif

/*
 * #define's for st_mode
 */

#ifndef S_IFMT		/* S_IFMT is not POSIX */
#define S_IFMT		0170000
#endif

#if !defined(S_ISCHR) && defined(S_IFCHR)
#define S_ISCHR(mode)	(((mode) & S_IFMT) == S_IFCHR)
#endif

#if !defined(S_ISBLK) && defined(S_IFBLK)
#define S_ISBLK(mode)	(((mode) & S_IFMT) == S_IFBLK)
#endif

/* SVR4 has symlinks (even S_IFLNK), but it lacks S_ISLNK() !! */
#if !defined(S_ISLNK) && defined (S_IFLNK)
#define S_ISLNK(mode)	(((mode) & S_IFMT) == S_IFLNK)
#endif

#ifndef S_ISUID
#define S_ISUID		04000
#endif
#ifndef S_ISGID
#define S_ISGID		02000
#endif
#ifndef S_ISVTX		/* S_ISVTX is not POSIX */
#define S_ISVTX		01000
#endif

#ifndef S_IRUSR
#define S_IRUSR		0400
#define S_IWUSR		0200
#define S_IXUSR		0100
#define S_IRGRP		0040
#define S_IWGRP		0020
#define S_IXGRP		0010
#define S_IROTH		0004
#define S_IWOTH		0002
#define S_IXOTH		0001
#endif

void stat_file(const char *file)
{
    struct stat inode;
    struct passwd *pw;
    struct group *gr;
    char path[PATH_MAX+1];    
    int n;

    if (! file)
	return;
    
    setpwent();
    setgrent();

    if (STAT(file, &inode) < 0) {
	perror(file);
	return;
    }

    printf("File: %s", file);
#ifdef	S_ISLNK	/* XXX */    
    if (S_ISLNK(inode.st_mode)) {
	memset(path, 0, PATH_MAX+1);
	n = readlink(file, path, PATH_MAX);
	if (n < 0) {
	    fprintf(stderr, "readlink(): ");
	    perror(file);
	}
	else {
	    printf(" -> %s", path);
	}
    }
#endif
    putchar('\n');    
    
    printf("Inode: %lu\n", (unsigned long) inode.st_ino);
    printf("Links: %lu\n", (unsigned long) inode.st_nlink);
    
    /* ID of device containing this file */
    printf("Device major: %lu, minor: %lu\n",
	   (unsigned long) major(inode.st_dev),
	   (unsigned long) minor(inode.st_dev));
/* XXX - st_rdev is not POSIX */
#if defined(S_ISCHR) || defined(S_ISBLK)    
    /* ID of device managed by the special file */
    if (S_ISCHR(inode.st_mode) || S_ISBLK(inode.st_mode)) {
	printf("Device type major: %lu, minor: %lu\n",
	       (unsigned long) major(inode.st_rdev),
	       (unsigned long) minor(inode.st_rdev));
    }
#endif    
    
    printf("Mode: 0%lo\t", (unsigned long) inode.st_mode);
    switch (inode.st_mode & S_IFMT) {
#ifdef S_IFDIR
    case S_IFDIR:
	putchar('d');	/* directory */
	break;
#endif
#ifdef S_IFCHR	
    case S_IFCHR:
	putchar('c');	/* character device */
	break;
#endif
#ifdef S_IFBLK
    case S_IFBLK:
	putchar('b');	/* block device */
	break;
#endif
#ifdef S_IFREG
    case S_IFREG:
	putchar('-');	/* regular file */
	break;
#endif    
#ifdef S_IFIFO
    case S_IFIFO:
	putchar('p');	/* fifo */
	break;
#endif	
#ifdef S_IFLNK
    case S_IFLNK:
	putchar('l');	/* symbolic link */
	break;
#endif
#ifdef S_IFSOCK
    case S_IFSOCK:
	putchar('s');	/* socket */
	break;
#endif
#ifdef S_IFWHT
    case S_IFWHT:
	putchar('w');	/* whiteout */
	break;
#endif	
    default:
	putchar('?');	/* Unknown */
    }

    if (inode.st_mode & S_IRUSR) putchar('r'); else putchar('-');
    if (inode.st_mode & S_IWUSR) putchar('w'); else putchar('-');
    if (inode.st_mode & S_IXUSR) {
	if (inode.st_mode & S_ISUID) putchar('s'); else putchar('x');
    }
    else {
	if (inode.st_mode & S_ISUID) putchar('S'); else putchar('-');
    }
    
    if (inode.st_mode & S_IRGRP) putchar('r'); else putchar('-');
    if (inode.st_mode & S_IWGRP) putchar('w'); else putchar('-');
    if (inode.st_mode & S_IXGRP) {
	if (inode.st_mode & S_ISGID) putchar('s'); else putchar('x');
    }
    else {
	if (inode.st_mode & S_ISGID) putchar('S'); else putchar('-');
    }

    if (inode.st_mode & S_IROTH) putchar('r'); else putchar('-');
    if (inode.st_mode & S_IWOTH) putchar('w'); else putchar('-');
    if (inode.st_mode & S_IXOTH) {
	if (inode.st_mode & S_ISVTX) putchar ('t'); else putchar('x');
    }
    else {
	if (inode.st_mode & S_ISVTX) putchar('T'); else putchar('-');
    }
    putchar('\n');
    
    printf("UID: %-10lu", (unsigned long) inode.st_uid);
    pw = getpwuid(inode.st_uid);
    if (pw && pw->pw_name) {
	printf(" %s\n", pw->pw_name);
    }
    else {
	printf(" **unknown**\n");
    }
	
    printf("GID: %-10lu", (unsigned long) inode.st_gid);
    gr = getgrgid(inode.st_gid);
    if (gr && gr->gr_name) {
	printf(" %s\n", gr->gr_name);
    }
    else {
	printf(" **unknown**\n");
    }

    printf("Size: %lu\n", (unsigned long) inode.st_size);

/* XXX - st_blocks & st_blksize are not POSIX */
#ifdef S_BLKSIZE
    printf("Blocks: %lu\n", (unsigned long) inode.st_blocks);
    printf("Optimal blocksize for I/O: %lu\n",
	   (unsigned long) inode.st_blksize);
#endif	
	
    printf("Last access  time: %s", ctime(&inode.st_atime));
    printf("Last modification: %s", ctime(&inode.st_mtime));
    printf("Last inode change: %s", ctime(&inode.st_ctime));
    
    endpwent();
    endgrent();

    return;
}

int main(int argc, char *argv[])
{
    if (argc != 2) {
	fprintf(stderr, "Usage: %s file\n", argv[0]);
	exit(1);
    }
    
    stat_file(argv[1]);
    
    exit(0);
}
	       
