/*
 * sysconf
 *    Uses sysconf(3) to get configuration information at run time
 *
 * (!c) 2002 by Ighighi
 * Venezuela
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

static void print_sysconf(const char *s, int name);

#define do_it(name)	print_sysconf(#name, name)

static void print_sysconf(const char *s, int name)
{
    long value;

    value = sysconf(name);
    if (value < 0) {
	if (errno) {
	    fprintf(stderr, "Error: ");
	    perror(s);
	}
	else {
	    printf("%-32s    ", s);	    
	    printf("%15s\n", "unlimited");
	}
    }
    else {
	printf("%-32s    ", s);	
	printf("%15ld\n", value);
    }
    
    return;
}

int main(void)
{

    printf("%-32s    %15s\n", "name", "value");    
    
#ifdef _SC_ARG_MAX
    do_it(_SC_ARG_MAX);
#endif

#ifdef _SC_CHILD_MAX
    do_it(_SC_CHILD_MAX);
#endif

#ifdef _SC_CLK_TCK
    do_it(_SC_CLK_TCK);
#endif

#ifdef _SC_NGROUPS_MAX
    do_it(_SC_NGROUPS_MAX);
#endif

#ifdef _SC_OPEN_MAX
    do_it(_SC_OPEN_MAX);
#endif

#ifdef _SC_STREAM_MAX
    do_it(_SC_STREAM_MAX);
#endif

#ifdef _SC_TZNAME_MAX
    do_it(_SC_TZNAME_MAX);
#endif

#ifdef _SC_JOB_CONTROL
    do_it(_SC_JOB_CONTROL);
#endif

#ifdef _SC_SAVED_IDS
    do_it(_SC_SAVED_IDS);
#endif

#ifdef _SC_REALTIME_SIGNALS
    do_it(_SC_REALTIME_SIGNALS);
#endif

#ifdef _SC_PRIORITY_SCHEDULING
    do_it(_SC_PRIORITY_SCHEDULING);
#endif

#ifdef _SC_TIMERS
    do_it(_SC_TIMERS);
#endif

#ifdef _SC_ASYNCHRONOUS_IO
    do_it(_SC_ASYNCHRONOUS_IO);
#endif

#ifdef _SC_PRIORITIZED_IO
    do_it(_SC_PRIORITIZED_IO);
#endif

#ifdef _SC_SYNCHRONIZED_IO
    do_it(_SC_SYNCHRONIZED_IO);
#endif

#ifdef _SC_FSYNC
    do_it(_SC_FSYNC);
#endif

#ifdef _SC_MAPPED_FILES
    do_it(_SC_MAPPED_FILES);
#endif

#ifdef _SC_MEMLOCK
    do_it(_SC_MEMLOCK);
#endif

#ifdef _SC_MEMLOCK_RANGE
    do_it(_SC_MEMLOCK_RANGE);
#endif

#ifdef _SC_MEMORY_PROTECTION
    do_it(_SC_MEMORY_PROTECTION);
#endif

#ifdef _SC_MESSAGE_PASSING
    do_it(_SC_MESSAGE_PASSING);
#endif

#ifdef _SC_SEMAPHORES
    do_it(_SC_SEMAPHORES);
#endif

#ifdef _SC_SHARED_MEMORY_OBJECTS
    do_it(_SC_SHARED_MEMORY_OBJECTS);
#endif

#ifdef _SC_AIO_LISTIO_MAX
    do_it(_SC_AIO_LISTIO_MAX);
#endif

#ifdef _SC_AIO_MAX
    do_it(_SC_AIO_MAX);
#endif

#ifdef _SC_AIO_PRIO_DELTA_MAX
    do_it(_SC_AIO_PRIO_DELTA_MAX);
#endif

#ifdef _SC_DELAYTIMER_MAX
    do_it(_SC_DELAYTIMER_MAX);
#endif

#ifdef _SC_MQ_OPEN_MAX
    do_it(_SC_MQ_OPEN_MAX);
#endif

#ifdef _SC_MQ_PRIO_MAX
    do_it(_SC_MQ_PRIO_MAX);
#endif

#ifdef _SC_VERSION
    do_it(_SC_VERSION);
#endif

#ifdef _SC_PAGESIZE
    do_it(_SC_PAGESIZE);
#endif

#ifdef _SC_RTSIG_MAX
    do_it(_SC_RTSIG_MAX);
#endif

#ifdef _SC_SEM_NSEMS_MAX
    do_it(_SC_SEM_NSEMS_MAX);
#endif

#ifdef _SC_SEM_VALUE_MAX
    do_it(_SC_SEM_VALUE_MAX);
#endif

#ifdef _SC_SIGQUEUE_MAX
    do_it(_SC_SIGQUEUE_MAX);
#endif

#ifdef _SC_TIMER_MAX
    do_it(_SC_TIMER_MAX);
#endif

#ifdef _SC_BC_BASE_MAX
    do_it(_SC_BC_BASE_MAX);
#endif

#ifdef _SC_BC_DIM_MAX
    do_it(_SC_BC_DIM_MAX);
#endif

#ifdef _SC_BC_SCALE_MAX
    do_it(_SC_BC_SCALE_MAX);
#endif

#ifdef _SC_BC_STRING_MAX
    do_it(_SC_BC_STRING_MAX);
#endif

#ifdef _SC_COLL_WEIGHTS_MAX
    do_it(_SC_COLL_WEIGHTS_MAX);
#endif

#ifdef _SC_EQUIV_CLASS_MAX
    do_it(_SC_EQUIV_CLASS_MAX);
#endif

#ifdef _SC_EXPR_NEST_MAX
    do_it(_SC_EXPR_NEST_MAX);
#endif

#ifdef _SC_LINE_MAX
    do_it(_SC_LINE_MAX);
#endif

#ifdef _SC_RE_DUP_MAX
    do_it(_SC_RE_DUP_MAX);
#endif

#ifdef _SC_CHARCLASS_NAME_MAX
    do_it(_SC_CHARCLASS_NAME_MAX);
#endif

#ifdef _SC_2_VERSION
    do_it(_SC_2_VERSION);
#endif

#ifdef _SC_2_C_BIND
    do_it(_SC_2_C_BIND);
#endif

#ifdef _SC_2_C_DEV
    do_it(_SC_2_C_DEV);
#endif

#ifdef _SC_2_FORT_DEV
    do_it(_SC_2_FORT_DEV);
#endif

#ifdef _SC_2_FORT_RUN
    do_it(_SC_2_FORT_RUN);
#endif

#ifdef _SC_2_SW_DEV
    do_it(_SC_2_SW_DEV);
#endif

#ifdef _SC_2_LOCALEDEF
    do_it(_SC_2_LOCALEDEF);
#endif

#ifdef _SC_PII
    do_it(_SC_PII);
#endif

#ifdef _SC_PII_XTI
    do_it(_SC_PII_XTI);
#endif

#ifdef _SC_PII_SOCKET
    do_it(_SC_PII_SOCKET);
#endif

#ifdef _SC_PII_INTERNET
    do_it(_SC_PII_INTERNET);
#endif

#ifdef _SC_PII_OSI
    do_it(_SC_PII_OSI);
#endif

#ifdef _SC_POLL
    do_it(_SC_POLL);
#endif

#ifdef _SC_SELECT
    do_it(_SC_SELECT);
#endif

#ifdef _SC_UIO_MAXIOV
    do_it(_SC_UIO_MAXIOV);
#endif

#ifdef _SC_IOV_MAX
    do_it(_SC_IOV_MAX);
#endif

#ifdef _SC_UIO_MAXIOV
    do_it(_SC_UIO_MAXIOV);
#endif

#ifdef _SC_PII_INTERNET_STREAM
    do_it(_SC_PII_INTERNET_STREAM);
#endif

#ifdef _SC_PII_INTERNET_DGRAM
    do_it(_SC_PII_INTERNET_DGRAM);
#endif

#ifdef _SC_PII_OSI_COTS
    do_it(_SC_PII_OSI_COTS);
#endif

#ifdef _SC_PII_OSI_CLTS
    do_it(_SC_PII_OSI_CLTS);
#endif

#ifdef _SC_PII_OSI_M
    do_it(_SC_PII_OSI_M);
#endif

#ifdef _SC_T_IOV_MAX
    do_it(_SC_T_IOV_MAX);
#endif

#ifdef _SC_THREADS
    do_it(_SC_THREADS);
#endif

#ifdef _SC_THREAD_SAFE_FUNCTIONS
    do_it(_SC_THREAD_SAFE_FUNCTIONS);
#endif

#ifdef _SC_GETGR_R_SIZE_MAX
    do_it(_SC_GETGR_R_SIZE_MAX);
#endif

#ifdef _SC_GETPW_R_SIZE_MAX
    do_it(_SC_GETPW_R_SIZE_MAX);
#endif

#ifdef _SC_LOGIN_NAME_MAX
    do_it(_SC_LOGIN_NAME_MAX);
#endif

#ifdef _SC_TTY_NAME_MAX
    do_it(_SC_TTY_NAME_MAX);
#endif

#ifdef _SC_THREAD_DESTRUCTOR_ITERATIONS
    do_it(_SC_THREAD_DESTRUCTOR_ITERATIONS);
#endif

#ifdef _SC_THREAD_KEYS_MAX
    do_it(_SC_THREAD_KEYS_MAX);
#endif

#ifdef _SC_THREAD_STACK_MIN
    do_it(_SC_THREAD_STACK_MIN);
#endif

#ifdef _SC_THREAD_THREADS_MAX
    do_it(_SC_THREAD_THREADS_MAX);
#endif

#ifdef _SC_THREAD_ATTR_STACKADDR
    do_it(_SC_THREAD_ATTR_STACKADDR);
#endif

#ifdef _SC_THREAD_ATTR_STACKSIZE
    do_it(_SC_THREAD_ATTR_STACKSIZE);
#endif

#ifdef _SC_THREAD_PRIORITY_SCHEDULING
    do_it(_SC_THREAD_PRIORITY_SCHEDULING);
#endif

#ifdef _SC_THREAD_PRIO_INHERIT
    do_it(_SC_THREAD_PRIO_INHERIT);
#endif

#ifdef _SC_THREAD_PRIO_PROTECT
    do_it(_SC_THREAD_PRIO_PROTECT);
#endif

#ifdef _SC_THREAD_PROCESS_SHARED
    do_it(_SC_THREAD_PROCESS_SHARED);
#endif

#ifdef _SC_NPROCESSORS_CONF
    do_it(_SC_NPROCESSORS_CONF);
#endif

#ifdef _SC_NPROCESSORS_ONLN
    do_it(_SC_NPROCESSORS_ONLN);
#endif

#ifdef _SC_PHYS_PAGES
    do_it(_SC_PHYS_PAGES);
#endif

#ifdef _SC_AVPHYS_PAGES
    do_it(_SC_AVPHYS_PAGES);
#endif

#ifdef _SC_ATEXIT_MAX
    do_it(_SC_ATEXIT_MAX);
#endif

#ifdef _SC_PASS_MAX
    do_it(_SC_PASS_MAX);
#endif

#ifdef _SC_XOPEN_VERSION
    do_it(_SC_XOPEN_VERSION);
#endif

#ifdef _SC_XOPEN_XCU_VERSION
    do_it(_SC_XOPEN_XCU_VERSION);
#endif

#ifdef _SC_XOPEN_UNIX
    do_it(_SC_XOPEN_UNIX);
#endif

#ifdef _SC_XOPEN_CRYPT
    do_it(_SC_XOPEN_CRYPT);
#endif

#ifdef _SC_XOPEN_ENH_I18N
    do_it(_SC_XOPEN_ENH_I18N);
#endif

#ifdef _SC_XOPEN_SHM
    do_it(_SC_XOPEN_SHM);
#endif

#ifdef _SC_2_CHAR_TERM
    do_it(_SC_2_CHAR_TERM);
#endif

#ifdef _SC_2_C_VERSION
    do_it(_SC_2_C_VERSION);
#endif

#ifdef _SC_2_UPE
    do_it(_SC_2_UPE);
#endif

#ifdef _SC_XOPEN_XPG2
    do_it(_SC_XOPEN_XPG2);
#endif

#ifdef _SC_XOPEN_XPG3
    do_it(_SC_XOPEN_XPG3);
#endif

#ifdef _SC_XOPEN_XPG4
    do_it(_SC_XOPEN_XPG4);
#endif

#ifdef _SC_CHAR_BIT
    do_it(_SC_CHAR_BIT);
#endif

#ifdef _SC_CHAR_MAX
    do_it(_SC_CHAR_MAX);
#endif

#ifdef _SC_CHAR_MIN
    do_it(_SC_CHAR_MIN);
#endif

#ifdef _SC_INT_MAX
    do_it(_SC_INT_MAX);
#endif

#ifdef _SC_INT_MIN
    do_it(_SC_INT_MIN);
#endif

#ifdef _SC_LONG_BIT
    do_it(_SC_LONG_BIT);
#endif

#ifdef _SC_WORD_BIT
    do_it(_SC_WORD_BIT);
#endif

#ifdef _SC_MB_LEN_MAX
    do_it(_SC_MB_LEN_MAX);
#endif

#ifdef _SC_NZERO
    do_it(_SC_NZERO);
#endif

#ifdef _SC_SSIZE_MAX
    do_it(_SC_SSIZE_MAX);
#endif

#ifdef _SC_SCHAR_MAX
    do_it(_SC_SCHAR_MAX);
#endif

#ifdef _SC_SCHAR_MIN
    do_it(_SC_SCHAR_MIN);
#endif

#ifdef _SC_SHRT_MAX
    do_it(_SC_SHRT_MAX);
#endif

#ifdef _SC_SHRT_MIN
    do_it(_SC_SHRT_MIN);
#endif

#ifdef _SC_UCHAR_MAX
    do_it(_SC_UCHAR_MAX);
#endif

#ifdef _SC_UINT_MAX
    do_it(_SC_UINT_MAX);
#endif

#ifdef _SC_ULONG_MAX
    do_it(_SC_ULONG_MAX);
#endif

#ifdef _SC_USHRT_MAX
    do_it(_SC_USHRT_MAX);
#endif

#ifdef _SC_NL_ARGMAX
    do_it(_SC_NL_ARGMAX);
#endif

#ifdef _SC_NL_LANGMAX
    do_it(_SC_NL_LANGMAX);
#endif

#ifdef _SC_NL_MSGMAX
    do_it(_SC_NL_MSGMAX);
#endif

#ifdef _SC_NL_NMAX
    do_it(_SC_NL_NMAX);
#endif

#ifdef _SC_NL_SETMAX
    do_it(_SC_NL_SETMAX);
#endif

#ifdef _SC_NL_TEXTMAX
    do_it(_SC_NL_TEXTMAX);
#endif

#ifdef _SC_XBS5_ILP32_OFF32
    do_it(_SC_XBS5_ILP32_OFF32);
#endif

#ifdef _SC_XBS5_ILP32_OFFBIG
    do_it(_SC_XBS5_ILP32_OFFBIG);
#endif

#ifdef _SC_XBS5_LP64_OFF64
    do_it(_SC_XBS5_LP64_OFF64);
#endif

#ifdef _SC_XBS5_LPBIG_OFFBIG
    do_it(_SC_XBS5_LPBIG_OFFBIG);
#endif

#ifdef _SC_XOPEN_LEGACY
    do_it(_SC_XOPEN_LEGACY);
#endif

#ifdef _SC_XOPEN_REALTIME
    do_it(_SC_XOPEN_REALTIME);
#endif

#ifdef _SC_XOPEN_REALTIME_THREADS
    do_it(_SC_XOPEN_REALTIME_THREADS);
#endif

#ifdef _SC_ADVISORY_INFO
    do_it(_SC_ADVISORY_INFO);
#endif

#ifdef _SC_BARRIERS
    do_it(_SC_BARRIERS);
#endif

#ifdef _SC_BASE
    do_it(_SC_BASE);
#endif

#ifdef _SC_C_LANG_SUPPORT
    do_it(_SC_C_LANG_SUPPORT);
#endif

#ifdef _SC_C_LANG_SUPPORT_R
    do_it(_SC_C_LANG_SUPPORT_R);
#endif

#ifdef _SC_CLOCK_SELECTION
    do_it(_SC_CLOCK_SELECTION);
#endif

#ifdef _SC_CPUTIME
    do_it(_SC_CPUTIME);
#endif

#ifdef _SC_THREAD_CPUTIME
    do_it(_SC_THREAD_CPUTIME);
#endif

#ifdef _SC_DEVICE_IO
    do_it(_SC_DEVICE_IO);
#endif

#ifdef _SC_DEVICE_SPECIFIC
    do_it(_SC_DEVICE_SPECIFIC);
#endif

#ifdef _SC_DEVICE_SPECIFIC_R
    do_it(_SC_DEVICE_SPECIFIC_R);
#endif

#ifdef _SC_FD_MGMT
    do_it(_SC_FD_MGMT);
#endif

#ifdef _SC_FIFO
    do_it(_SC_FIFO);
#endif

#ifdef _SC_PIPE
    do_it(_SC_PIPE);
#endif

#ifdef _SC_FILE_ATTRIBUTES
    do_it(_SC_FILE_ATTRIBUTES);
#endif

#ifdef _SC_FILE_LOCKING
    do_it(_SC_FILE_LOCKING);
#endif

#ifdef _SC_FILE_SYSTEM
    do_it(_SC_FILE_SYSTEM);
#endif

#ifdef _SC_MONOTONIC_CLOCK
    do_it(_SC_MONOTONIC_CLOCK);
#endif

#ifdef _SC_MULTI_PROCESS
    do_it(_SC_MULTI_PROCESS);
#endif

#ifdef _SC_SINGLE_PROCESS
    do_it(_SC_SINGLE_PROCESS);
#endif

#ifdef _SC_NETWORKING
    do_it(_SC_NETWORKING);
#endif

#ifdef _SC_READER_WRITER_LOCKS
    do_it(_SC_READER_WRITER_LOCKS);
#endif

#ifdef _SC_SPIN_LOCKS
    do_it(_SC_SPIN_LOCKS);
#endif

#ifdef _SC_REGEXP
    do_it(_SC_REGEXP);
#endif

#ifdef _SC_REGEX_VERSION
    do_it(_SC_REGEX_VERSION);
#endif

#ifdef _SC_SHELL
    do_it(_SC_SHELL);
#endif

#ifdef _SC_SIGNALS
    do_it(_SC_SIGNALS);
#endif

#ifdef _SC_SPAWN
    do_it(_SC_SPAWN);
#endif

#ifdef _SC_SPORADIC_SERVER
    do_it(_SC_SPORADIC_SERVER);
#endif

#ifdef _SC_THREAD_SPORADIC_SERVER
    do_it(_SC_THREAD_SPORADIC_SERVER);
#endif

#ifdef _SC_SYSTEM_DATABASE
    do_it(_SC_SYSTEM_DATABASE);
#endif

#ifdef _SC_SYSTEM_DATABASE_R
    do_it(_SC_SYSTEM_DATABASE_R);
#endif

#ifdef _SC_TIMEOUTS
    do_it(_SC_TIMEOUTS);
#endif

#ifdef _SC_TYPED_MEMORY_OBJECTS
    do_it(_SC_TYPED_MEMORY_OBJECTS);
#endif

#ifdef _SC_USER_GROUPS
    do_it(_SC_USER_GROUPS);
#endif

#ifdef _SC_USER_GROUPS_R
    do_it(_SC_USER_GROUPS_R);
#endif

#ifdef _SC_2_PBS
    do_it(_SC_2_PBS);
#endif

#ifdef _SC_2_PBS_ACCOUNTING
    do_it(_SC_2_PBS_ACCOUNTING);
#endif

#ifdef _SC_2_PBS_LOCATE
    do_it(_SC_2_PBS_LOCATE);
#endif

#ifdef _SC_2_PBS_MESSAGE
    do_it(_SC_2_PBS_MESSAGE);
#endif

#ifdef _SC_2_PBS_TRACK
    do_it(_SC_2_PBS_TRACK);
#endif

#ifdef _SC_SYMLOOP_MAX
    do_it(_SC_SYMLOOP_MAX);
#endif

#ifdef _SC_STREAMS
    do_it(_SC_STREAMS);
#endif

#ifdef _SC_2_PBS_CHECKPOINT
    do_it(_SC_2_PBS_CHECKPOINT);
#endif

#ifdef _SC_V6_ILP32_OFF32
    do_it(_SC_V6_ILP32_OFF32);
#endif

#ifdef _SC_V6_ILP32_OFFBIG
    do_it(_SC_V6_ILP32_OFFBIG);
#endif

#ifdef _SC_V6_LP64_OFF64
    do_it(_SC_V6_LP64_OFF64);
#endif

#ifdef _SC_V6_LPBIG_OFFBIG
    do_it(_SC_V6_LPBIG_OFFBIG);
#endif

#ifdef _SC_HOST_NAME_MAX
    do_it(_SC_HOST_NAME_MAX);
#endif

#ifdef _SC_TRACE
    do_it(_SC_TRACE);
#endif

#ifdef _SC_TRACE_EVENT_FILTER
    do_it(_SC_TRACE_EVENT_FILTER);
#endif

#ifdef _SC_TRACE_INHERIT
    do_it(_SC_TRACE_INHERIT);
#endif

#ifdef _SC_TRACE_LOG
    do_it(_SC_TRACE_LOG);
#endif

    return 0;
}
